# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Dataclasses user models to generate table rows."""

from dataclasses import InitVar, dataclass


@dataclass
class User:
    """User dataclass model."""

    id: int
    created: str
    updated: str
    username: str
    displayname: str
    email: str
    password: str
    active: bool
    confirmed_at: str
    version_id: int
    profile: dict
    preferences: dict

    _table_name: InitVar[str] = "accounts_user"


@dataclass
class LoginInformation:
    """Login information dataclass model."""

    user_id: int
    last_login_at: str
    current_login_at: str
    last_login_ip: str
    current_login_ip: str
    login_count: int

    _table_name: InitVar[str] = "accounts_user_login_information"


@dataclass
class SessionActivity:
    """Session activity dataclass model."""

    user_id: int
    created: str
    updated: str
    sid_s: str
    ip: str
    country: str
    browser: str
    browser_version: str
    os: str
    device: str

    _table_name: InitVar[str] = "accounts_user_session_activity"


@dataclass
class Tokens:
    """OAuth server tokens dataclass model."""

    id: int
    client_id: str
    user_id: int
    token_type: str
    access_token: bytes
    refresh_token: bytes
    expires: str
    _scopes: str
    is_personal: bool
    is_internal: bool

    _table_name: InitVar[str] = "oauth2server_token"


@dataclass
class UserIdentity:
    """User identity dataclass model."""

    created: str
    updated: str
    id: str
    method: str
    id_user: int

    _table_name: InitVar[str] = "accounts_useridentity"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

import tempfile

import pytest


@pytest.fixture(scope="function")
def tmp_dir():
    """Yields a temporary directory."""
    tmp_dir = tempfile.TemporaryDirectory()
    yield tmp_dir
    tmp_dir.cleanup()


@pytest.fixture(scope="module")
def parent_cache():
    """Records parent cache.

    Keys are concept recids and values are UUIDs.
    """
    return {"123456": "12345678-abcd-1a2b-3c4d-123abc456def"}


@pytest.fixture(scope="module")
def communities_cache():
    """Communities cache.

    Keys are community slugs and values are UUIDs.
    """
    return {"comm": "12345678-abcd-1a2b-3c4d-123abc456def"}


@pytest.fixture(scope="module")
def cache(parent_cache, communities_cache):
    """Global cache containing the other ones."""
    return {
        "parents": parent_cache,
        "communities": communities_cache,
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio RDM migration user table load module."""
import random

from ...load.postgresql import TableGenerator, generate_uuid
from .models import Community, CommunityMember, FeaturedCommunity


def _generate_members_uuids(data):
    for member in data["community_members"]:
        member["id"] = generate_uuid(data)
    return data["community_members"]


def _generate_featured_community_id(data):
    if not data.get("featured_community"):
        return None
    return _pid_pk(data)


# keep track of generated PKs, since there's a chance they collide
GENERATED_PID_PKS = set()


def _pid_pk(data):
    """Generates a random integer between 1M and 2B, avoiding duplication with previously generated numbers."""
    lower_limit = 1_000_000
    upper_limit = 2_147_483_647 - 1

    while True:
        # we start at 1M to avoid collisions with existing low-numbered PKs
        val = random.randint(lower_limit, upper_limit)
        if val not in GENERATED_PID_PKS:
            GENERATED_PID_PKS.add(val)
            return val


class CommunityTableGenerator(TableGenerator):
    """User and related tables load."""

    def __init__(self, communities_cache):
        """Constructor."""
        self.communities_cache = communities_cache
        super().__init__(
            tables=[Community, CommunityMember, FeaturedCommunity],
            pks=[
                ("community.id", generate_uuid),
                ("community_members", _generate_members_uuids),
                ("featured_community.id", _generate_featured_community_id),
            ],
        )

    def _generate_rows(self, data, **kwargs):
        community = data["community"]
        community_id = community["id"]
        community_slug = community["slug"]
        if self.communities_cache.get(community_slug) is None:
            self.communities_cache[community_slug] = community_id

        yield Community(**community)

        community_members = data["community_members"]
        for member in community_members:
            member["community_id"] = community_id
            yield CommunityMember(**member)

        featured_community = data["featured_community"]
        if featured_community.get("id"):
            featured_community["community_id"] = community_id
            yield FeaturedCommunity(**featured_community)

    def cleanup(self, **kwargs):
        """Cleanup."""
        pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""InvenioRDM migration streams runner."""

import yaml

from .streams import Stream


class Runner:
    """ETL streams runner."""

    def _read_config(self, filepath):
        """Read config from file."""
        with open(filepath) as f:
            return yaml.safe_load(f)

    def __init__(self, stream_definitions, config_filepath):
        """Constructor."""
        self.streams = []
        self.cache = {
            # used to keep track of what Parent IDs we've already inserted in the PIDs table.
            # {
            #     '<parent_pid>': {
            #         'id': <generated_parent_uuid>,
            #         'version': {
            #             'latest_index': 'record_index',
            #             'latest_id': 'record id',
            #         }
            # }
            "parent": {},
            "communities": {},
        }
        config = self._read_config(config_filepath)

        for definition in stream_definitions:
            stream_config = config.get(definition.name)
            if stream_config is not None:
                # merge cache objects from stream definition config
                stream_cache = stream_config.get("load", {}).pop("cache", {})
                self.cache.update(stream_cache)
                self.streams.append(
                    Stream(
                        definition.name,
                        definition.extract_cls(**stream_config.get("extract", {})),
                        definition.transform_cls(**stream_config.get("transform", {})),
                        definition.load_cls(
                            **stream_config.get("load", {}), cache=self.cache
                        ),
                    )
                )

    def run(self):
        """Run ETL streams."""
        for stream in self.streams:
            stream.run()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Requests load tests."""

import os
from unittest.mock import patch

import pytest

from invenio_rdm_migrator.streams.requests import RequestCopyLoad


@pytest.fixture(scope="function")
def request_copy_load(cache, tmp_dir):
    """Request load instance."""
    # the db queries will be mocked
    load = RequestCopyLoad(cache, "None", tmp_dir.name)
    yield load
    load._cleanup()


@patch(
    "invenio_rdm_migrator.load.postgresql.uuid4",
    lambda: "12345678-abcd-1a2b-3c4d-123abc456def",
)
def test_request_load_prepare(request_copy_load, transformed_incl_req_entry):
    """Test the table preparation (file creation)."""
    tables = list(request_copy_load._prepare([transformed_incl_req_entry]))

    # assert tables
    assert len(tables) == 1
    assert tables[0]._table_name == "request_metadata"

    # assert files were created and have the content
    files = list(os.scandir(request_copy_load.tmp_dir))
    assert len(files) == 1

    # assert request metadata content
    with open(f"{request_copy_load.tmp_dir}/request_metadata.csv", "r") as file:
        # uuid, json, created, updated, version_id, number, expired_at
        expected = (
            "12345678-abcd-1a2b-3c4d-123abc456def,"
            '"{""type"": ""community-inclusion"", ""title"": ""title"", ""topic"": {""record"": ""123456""}, ""status"": ""submitted"", ""receiver"": {""community"": ""12345678-abcd-1a2b-3c4d-123abc456def""}, ""created_by"": {""user"": ""3""}, ""$schema"": ""local://requests/request-v1.0.0.json""}"'
            ",01/01/2023,01/01/2023,1,1,01/01/2023\n"
        )
        content = file.read()
        assert content == expected



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Requests load tests."""

import os
from unittest.mock import patch

import pytest

from invenio_rdm_migrator.streams.files import FilesCopyLoad


@pytest.fixture(scope="function")
def files_copy_load(tmp_dir):
    """Request load instance."""
    # the db queries will be mocked
    load = FilesCopyLoad(cache={}, db_uri="None", tmp_dir=tmp_dir.name)
    yield load
    load._cleanup()


@patch(
    "invenio_rdm_migrator.load.postgresql.uuid4",
    lambda: "12345678-abcd-1a2b-3c4d-123abc456def",
)
def test_files_load_prepare(files_copy_load, transformed_files_entry):
    """Test the table preparation (file creation)."""
    tables = list(files_copy_load._prepare([transformed_files_entry]))

    # assert tables
    assert len(tables) == 3
    assert tables[0]._table_name == "files_files"
    assert tables[1]._table_name == "files_bucket"
    assert tables[2]._table_name == "files_object"

    # assert files were created and have the content
    files = list(os.scandir(files_copy_load.tmp_dir))
    assert len(files) == 3

    # assert files bucket content
    with open(f"{files_copy_load.tmp_dir}/files_bucket.csv", "r") as file:
        # id, created, updated, default_location, default_storage_class, size, quota_size, max_file_size,locked, deleted
        expected = (
            "1," "2023-04-19," "2023-04-19," "1," "L," "1234," "," "," "False," "False"
        )
        content = file.read().rstrip()
        assert content == expected
    # assert files object version content
    with open(f"{files_copy_load.tmp_dir}/files_files.csv", "r") as file:
        # id, created, updated, uri, storage_class, size, checksum, readable, writable, last_check_at, last_check
        expected = (
            "3,"
            "2023-04-19,"
            "2023-04-19,"
            "path/to/file,"
            "L,"
            "1234,"
            "md5:abcd1234,"
            "True,"
            "True,"
            ","
            "False"
        )
        content = file.read().rstrip()
        assert content == expected
    # assert files instance content
    with open(f"{files_copy_load.tmp_dir}/files_object.csv", "r") as file:
        #  version_id, created, updated, key, bucket_id, file_id, _mimetype, is_head
        expected = "2," "2023-04-19," "2023-04-19," "file.txt," "1," "3," "," "True"
        content = file.read().rstrip()
        assert content == expected



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""PostgreSQL load module."""


import contextlib
import csv
import json
from abc import ABC, abstractmethod
from dataclasses import fields
from datetime import datetime
from pathlib import Path
from uuid import UUID, uuid4

import psycopg
from invenio_records.dictutils import dict_set  # TODO: can we do without?

from .base import Load


def _ts(iso=True):
    """Current timestamp string."""
    dt = datetime.now()
    return dt.isoformat() if iso else dt.timestamp()


def as_csv_row(dc):
    """Serialize a dataclass instance as a CSV-writable row."""
    row = []
    for f in fields(dc):
        val = getattr(dc, f.name)
        if val:
            if issubclass(f.type, (dict,)):
                val = json.dumps(val)
            elif issubclass(f.type, (datetime,)):
                val = val.isoformat()
            elif issubclass(f.type, (UUID,)):
                val = str(val)
        row.append(val)
    return row


def generate_uuid(data):
    """Generate a UUID."""
    return str(uuid4())


class PostgreSQLCopyLoad(Load):  # TODO: abstract SQL from PostgreSQL?
    """PostgreSQL COPY load."""

    def __init__(self, db_uri, table_loads, tmp_dir):
        """Constructor."""
        self.db_uri = db_uri
        self.tmp_dir = Path(tmp_dir) / f"tables{_ts(iso=False)}"
        self.table_loads = table_loads

    def _cleanup(self, db=False):
        """Cleanup csv files and DB after load."""
        for table in self.table_loads:
            table.cleanup(db=db)

    def _prepare(self, entries):
        """Dump entries in csv files for COPY command."""
        self.tmp_dir.mkdir(parents=True, exist_ok=True)

        _prepared_tables = []
        for table in self.table_loads:
            # otherwise the generator is exhausted by the first table
            # TODO: nested generators, how expensive is this copy op?
            _prepared_tables.extend(table.prepare(self.tmp_dir, entries=entries))

        return iter(_prepared_tables)  # yield at the end vs yield per table

    def _load(self, table_entries):
        """Bulk load CSV table files.

        Loads the tables in the order given by the generator.
        """
        with psycopg.connect(self.db_uri) as conn:
            for table in table_entries:
                name = table._table_name
                cols = ", ".join([f.name for f in fields(table)])
                fpath = self.tmp_dir / f"{name}.csv"
                if fpath.exists():
                    # total file size for progress logging
                    file_size = fpath.stat().st_size

                    print(f"[{_ts()}] COPY FROM {fpath}")  # TODO: logging
                    with contextlib.ExitStack() as stack:
                        cur = stack.enter_context(conn.cursor())
                        copy = stack.enter_context(
                            cur.copy(f"COPY {name} ({cols}) FROM STDIN (FORMAT csv)")
                        )
                        fp = stack.enter_context(open(fpath, "r"))

                        block_size = 8192

                        def _data_blocks(block_size=8192):
                            data = fp.read(block_size)
                            while data:
                                yield data
                                data = fp.read(block_size)

                        for idx, block in enumerate(_data_blocks(block_size)):
                            if idx % 100:
                                cur_bytes = idx * block_size
                                percentage = (cur_bytes / file_size) * 100
                                progress = (
                                    f"{cur_bytes}/{file_size} ({percentage:.2f}%)"
                                )
                                print(f"[{_ts()}] {name}: {progress}")
                            copy.write(block)
                else:
                    # FIXME: log a WARNING/ERROR
                    print(f"[{_ts()}] {name}: no data to load")
                conn.commit()

    def run(self, entries, cleanup=False):
        """Load entries."""
        table_entries = self._prepare(entries)
        self._load(table_entries)

        if cleanup:
            self._cleanup()


class TableGenerator(ABC):
    """Create CSV files with table create and inserts."""

    def __init__(self, tables, pks=None):
        """Constructor."""
        self._tables = tables
        self.pks = pks or []

    @abstractmethod
    def _generate_rows(self, **kwargs):
        """Yield generated rows."""
        pass

    @abstractmethod
    def cleanup(self, **kwargs):
        """Cleanup."""
        pass

    def _generate_pks(self, data):
        for path, pk_func in self.pks:
            dict_set(data, path, pk_func(data))

    def _resolve_references(self, data, **kwargs):
        """Resolve references e.g communities slug names."""
        pass

    def prepare(self, tmp_dir, entries, **kwargs):
        """Compute rows."""
        # use this context manager to close all opened files at once
        with contextlib.ExitStack() as stack:
            out_files = {}
            for entry in entries:
                # is_db_empty would come in play and make _generate_pks optional
                self._generate_pks(entry)
                # resolve entry references
                self._resolve_references(entry)
                for entry in self._generate_rows(entry):
                    if entry._table_name not in out_files:
                        fpath = tmp_dir / f"{entry._table_name}.csv"
                        out_files[entry._table_name] = csv.writer(
                            stack.enter_context(open(fpath, "w+"))
                        )
                    writer = out_files[entry._table_name]
                    writer.writerow(as_csv_row(entry))

        return self._tables



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio RDM migration requests load module."""

from ...load import PostgreSQLCopyLoad
from .table_generator import RequestTableGenerator


class RequestCopyLoad(PostgreSQLCopyLoad):
    """PostgreSQL COPY load."""

    def __init__(self, cache, db_uri, tmp_dir):
        """Constructor."""
        self.communities_cache = cache.get("communities", {})
        super().__init__(
            db_uri=db_uri,
            tmp_dir=tmp_dir,
            table_loads=[
                RequestTableGenerator(self.communities_cache),
            ],
        )

    def _validate(self):
        """Validate data before loading."""
        pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio RDM migration record table load module."""


import random
from datetime import datetime

from ...load.models import PersistentIdentifier
from ...load.postgresql import TableGenerator, generate_uuid
from ..communities.models import RDMParentCommunityMetadata
from .models import RDMParentMetadata, RDMRecordFile, RDMRecordMetadata, RDMVersionState


class RDMVersionStateTableGenerator(TableGenerator):
    """RDM version state computed table."""

    def __init__(self, parent_cache):
        """Constructor."""
        super().__init__(tables=[RDMVersionState])
        self.parent_cache = parent_cache

    def _generate_rows(self, parent_entry, **kwargs):
        # Version state to be populated in the end from the final state
        yield RDMVersionState(
            latest_index=parent_entry["version"]["latest_index"],
            parent_id=parent_entry["id"],
            latest_id=parent_entry["version"]["latest_id"],
            next_draft_id=None,
        )

    def prepare(self, tmp_dir, entries, **kwargs):
        """Overwrite entries with parent cache entries."""
        return super().prepare(tmp_dir, self.parent_cache.values(), **kwargs)

    def cleanup(self, **kwargs):
        """Cleanup."""
        pass


# keep track of generated PKs, since there's a chance they collide
GENERATED_PID_PKS = set()


def _pid_pk():
    while True:
        # we start at 1M to avoid collisions with existing low-numbered PKs
        val = random.randint(1_000_000, 2_147_483_647 - 1)
        if val not in GENERATED_PID_PKS:
            GENERATED_PID_PKS.add(val)
            return val


def _generate_recid(data):
    return {
        "pk": _pid_pk(),
        "obj_type": "rec",
        "pid_type": "recid",
        "status": "R",
    }


def generate_files_uuids(data):
    """Generate uuid for every file in the list.

    Return the transformed list with the generated ids.
    """
    _files = data["record_files"]
    for _file in _files:
        _file["id"] = generate_uuid({})
    return _files


def generate_record_uuid(data):
    """Generate record uuid if not present."""
    _id = data.get("id")
    return _id if _id else generate_uuid(None)


class RDMRecordTableGenerator(TableGenerator):
    """RDM Record and related tables load."""

    def __init__(self, parent_cache, communities_cache):
        """Constructor."""
        super().__init__(
            tables=[
                PersistentIdentifier,
                RDMParentMetadata,
                RDMRecordMetadata,
                RDMParentCommunityMetadata,
                RDMRecordFile,
            ],
            pks=[
                ("record.id", generate_record_uuid),
                ("parent.id", generate_uuid),
                ("record.json.pid", _generate_recid),
                ("parent.json.pid", _generate_recid),
                ("record.parent_id", lambda d: d["parent"]["id"]),
                ("record_files", generate_files_uuids),
            ],
        )
        self.parent_cache = parent_cache
        self.communities_cache = communities_cache

    def _generate_rows(self, data, **kwargs):
        now = datetime.utcnow().isoformat()

        # record
        rec = data["record"]
        record_pid = rec["json"]["pid"]
        parent = data["parent"]
        rec_parent_id = self.parent_cache.get(parent["json"]["id"], {}).get("id")
        yield RDMRecordMetadata(
            id=rec["id"],
            json=rec["json"],
            created=rec["created"],
            updated=rec["updated"],
            version_id=rec["version_id"],
            index=rec["index"],
            bucket_id=rec.get("bucket_id"),
            parent_id=rec_parent_id or rec["parent_id"],
        )
        # recid
        yield PersistentIdentifier(
            id=record_pid["pk"],
            pid_type=record_pid["pid_type"],
            pid_value=rec["json"]["id"],
            status=record_pid["status"],
            object_type=record_pid["obj_type"],
            object_uuid=rec["id"],
            created=now,
            updated=now,
        )
        # DOI
        if "doi" in rec["json"]["pids"]:
            yield PersistentIdentifier(
                id=_pid_pk(),
                pid_type="doi",
                pid_value=rec["json"]["pids"]["doi"]["identifier"],
                status="R",
                object_type="rec",
                object_uuid=rec["id"],
                created=now,
                updated=now,
            )
        # OAI
        if "oai" in rec["json"]["pids"]:
            yield PersistentIdentifier(
                id=_pid_pk(),
                pid_type="oai",
                pid_value=rec["json"]["pids"]["oai"]["identifier"],
                status="R",
                object_type="rec",
                object_uuid=rec["id"],
                created=now,
                updated=now,
            )

        # parent
        if parent["json"]["id"] not in self.parent_cache:
            self.parent_cache[parent["json"]["id"]] = dict(
                id=parent["id"],
                version=dict(latest_index=rec["index"], latest_id=rec["id"]),
            )
            parent_pid = parent["json"]["pid"]
            # record
            yield RDMParentMetadata(
                id=parent["id"],
                json=parent["json"],
                created=parent["created"],
                updated=parent["updated"],
                version_id=parent["version_id"],
            )
            # recid
            yield PersistentIdentifier(
                id=parent_pid["pk"],
                pid_type=parent_pid["pid_type"],
                pid_value=parent["json"]["id"],
                status=parent_pid["status"],
                object_type=parent_pid["obj_type"],
                object_uuid=parent["id"],
                created=now,
                updated=now,
            )
            # parent community
            if "default" in parent["json"]["communities"]:
                yield RDMParentCommunityMetadata(
                    community_id=parent["json"]["communities"]["default"],
                    record_id=parent["id"],
                    request_id=None,
                )
        else:
            # parent in cache - update version
            cached_parent = self.parent_cache[parent["json"]["id"]]
            # check if current record is a new version of the cached one
            if cached_parent["version"]["latest_index"] < rec["index"]:
                cached_parent["version"] = dict(
                    latest_index=rec["index"], latest_id=rec["id"]
                )
        # record files
        record_files = data["record_files"]
        for _file in record_files:
            yield RDMRecordFile(
                id=_file["id"],
                json=_file["json"],
                created=_file["created"],
                updated=_file["updated"],
                version_id=_file["version_id"],
                key=_file["key"],
                record_id=rec["id"],
                object_version_id=_file["object_version_id"],
            )

    def _resolve_references(self, data, **kwargs):
        """Resolve references e.g communities slug names."""

        def _resolve_communities(communities):
            default_slug = communities.get("default")
            default_id = self.communities_cache.get(default_slug)
            if not default_id:
                # TODO: maybe raise error without correct default community?
                communities = {}
            communities["default"] = default_id

            communities_slugs = communities.get("ids", [])
            _ids = []
            for slug in communities_slugs:
                _id = self.communities_cache.get(slug)
                if _id:
                    _ids.append(_id)
            communities["ids"] = _ids

        # resolve parent communities slug
        parent = data["parent"]
        communities = parent["json"].get("communities")
        if communities:
            _resolve_communities(communities)

    def _cleanup_db(self):
        """Cleanup DB after load."""
        # FIXME: abstract to tables, can we do without invenio imports
        # cant fix atm, versions state needs to be deleted in the middle
        # from invenio_db import db
        # from invenio_pidstore.models import PersistentIdentifier
        # from invenio_rdm_records.records.models import (
        #     RDMDraftMetadata,
        #     RDMParentMetadata,
        #     RDMRecordMetadata,
        #     RDMVersionsState,
        # )
        # PersistentIdentifier.query.filter(
        #     PersistentIdentifier.pid_type.in_(("recid", "doi", "oai")),
        #     PersistentIdentifier.object_type == "rec",
        # ).delete()
        # RDMVersionsState.query.delete()
        # RDMRecordMetadata.query.delete()
        # RDMParentMetadata.query.delete()
        # RDMDraftMetadata.query.delete()
        # db.session.commit()
        pass

    def _cleanup_files(self):
        """Cleanup files after load."""
        # FIXME: tables does not return the name correct
        # delegate to table_loads?
        # for table in self.tables:
        #     fpath = self.tmp_dir / f"{table._table_name}.csv"
        #     print(f"Checking {fpath}")
        #     if fpath.exists():
        #         print(f"Deleting {fpath}")
        #         fpath.unlink(missing_ok=True)
        pass

    def cleanup(self, db, **kwargs):
        """Cleanup."""
        self._cleanup_files()

        if db:  # DB cleanup is not always desired
            self._cleanup_db()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio RDM migration load module."""

from .base import Load
from .postgresql import PostgreSQLCopyLoad, TableGenerator

__all__ = (
    "Load",
    "PostgreSQLCopyLoad",
    "TableGenerator",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Dataclasses models to generate table rows."""

from dataclasses import InitVar, dataclass


@dataclass
class PersistentIdentifier:
    """Persistent identifier dataclass model."""

    id: str
    pid_type: str
    pid_value: str
    status: str
    object_type: str
    object_uuid: str
    created: str
    updated: str

    _table_name: InitVar[str] = "pidstore_pid"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Dataclasses user models to generate table rows."""

from dataclasses import InitVar, dataclass


@dataclass
class FilesBucket:
    """Files bucket dataclass model."""

    id: str
    created: str
    updated: str
    default_location: str
    default_storage_class: str
    size: int
    quota_size: int
    max_file_size: int
    locked: bool
    deleted: bool

    _table_name: InitVar[str] = "files_bucket"


@dataclass
class FilesInstance:
    """Files instance dataclass model."""

    id: str
    created: str
    updated: str
    uri: str
    storage_class: str
    size: int
    checksum: str
    readable: bool
    writable: bool
    last_check_at: str
    last_check: bool

    _table_name: InitVar[str] = "files_files"


@dataclass
class FilesObjectVersion:
    """Files object version dataclass model."""

    version_id: str
    created: str
    updated: str
    key: str
    bucket_id: str
    file_id: str
    _mimetype: str
    is_head: bool

    _table_name: InitVar[str] = "files_object"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio RDM migration transform interfaces."""

from abc import ABC, abstractmethod


class Transform(ABC):
    """Base class for data transformation."""

    @abstractmethod
    def _transform(self, entry):
        """Transform entry."""
        pass

    def run(self, entries):
        """Transform and yield one element at a time."""
        for entry in entries:
            yield self._transform(entry)


class Entry(ABC):
    """Base entry class."""

    @abstractmethod
    def transform(self, entry):
        """Transform entry."""
        pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Dataclasses user models to generate table rows."""

from dataclasses import InitVar, dataclass


@dataclass
class Community:
    """Community dataclass model."""

    id: str
    created: str
    updated: str
    json: dict
    version_id: int
    slug: str
    bucket_id: str

    _table_name: InitVar[str] = "communities_metadata"


@dataclass
class RDMParentCommunityMetadata:
    """RDM Community Parent Metadata dataclass model."""

    community_id: str
    record_id: str
    request_id: str

    _table_name: InitVar[str] = "rdm_parents_community"


@dataclass
class CommunityMember:
    """Community members dataclass model."""

    id: str
    created: str
    updated: str
    json: dict
    version_id: int
    role: str
    visible: bool
    active: bool
    community_id: str
    user_id: int
    group_id: int
    request_id: int

    _table_name: InitVar[str] = "communities_members"


@dataclass
class FeaturedCommunity:
    """Featured community dataclass model."""

    community_id: str
    id: str
    created: str
    updated: str
    start_date: str

    _table_name: InitVar[str] = "communities_featured"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Migrator is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Dataclasses record models to generate table rows."""

from dataclasses import InitVar, dataclass


@dataclass
class RDMRecordMetadata:
    """RDM Record Metadata dataclass model."""

    id: str
    json: dict
    created: str
    updated: str
    version_id: int
    index: int
    bucket_id: str
    parent_id: str

    _table_name: InitVar[str] = "rdm_records_metadata"


@dataclass
class RDMParentMetadata:
    """RDM Parent Metadata dataclass model."""

    id: str
    json: dict
    created: str
    updated: str
    version_id: int

    _table_name: InitVar[str] = "rdm_parents_metadata"


@dataclass
class RDMVersionState:
    """RDM Version State dataclass model."""

    latest_index: int
    parent_id: str
    latest_id: str
    next_draft_id: str

    _table_name: InitVar[str] = "rdm_versions_state"


@dataclass
class RDMDraftMetadata:
    """RDM Draft Metadata dataclass model."""

    id: str
    json: dict
    created: str
    updated: str
    version_id: int
    index: int
    bucket_id: str
    parent_id: str

    expires_at: str
    fork_version_id: int

    _table_name: InitVar[str] = "rdm_drafts_metadata"


@dataclass
class RDMRecordFile:
    """RDM Record File dataclass model."""

    id: str
    json: dict
    created: str
    updated: str
    version_id: int
    key: str
    record_id: str
    object_version_id: str

    _table_name: InitVar[str] = "rdm_records_files"


@dataclass
class RDMDraftFile(RDMRecordFile):
    """RDM Draft File dataclass model."""

    _table_name: InitVar[str] = "rdm_drafts_files"
